var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const rollup_1 = require("rollup");
const child_process_1 = require("child_process");
const DevtoolHelper_1 = require("./util/DevtoolHelper");
const path_1 = require("./config/path");
const babel = require('rollup-plugin-babel');
const nodeResolve = require('rollup-plugin-node-resolve');
const commonjs = require('rollup-plugin-commonjs');
const typescript = require('rollup-plugin-typescript2');
const terser = require('rollup-plugin-terser').terser;
const minify = process.env.NODE_ENV === "production";
const config = require(path_1.paths.appEnvoyRollupConfig);
/**
 * Returns rollup plugins to be used
 */
function getRollupPlugins() {
    const rollupPlugins = [
        babel({
            exclude: 'node_modules/**',
        }),
        typescript({
            exclude: 'node_modules/**',
            rollupCommonJSResolveHack: true,
            clean: true,
        }),
        nodeResolve({
            module: true,
            main: true
        }),
        commonjs({
            include: 'node_modules/**'
        }),
    ];
    if (minify) {
        rollupPlugins.push(terser());
    }
    return rollupPlugins;
}
/**
 * Generate both `es` and `umd` format for a given bundle
 *
 * @param bundle - bundle to be handled
 */
function handleBundle(bundle) {
    return __awaiter(this, void 0, void 0, function* () {
        let umd;
        const code = (yield bundle.generate({
            format: 'es',
            strict: false,
        })).code;
        if (config.umdModuleName) {
            umd = (yield bundle.generate({
                format: 'umd',
                strict: false,
                name: config.umdModuleName,
            })).code;
        }
        return {
            code,
            umd,
        };
    });
}
/**
 * Clean up `dist` directory
 */
function clean() {
    child_process_1.exec('rm -rf dist && mkdir dist');
}
function buildEnvoyPackage() {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            clean();
            const plugins = getRollupPlugins();
            const rollupOptions = Object.assign({}, {
                input: config.input,
                output: config.output,
                cache: config.cache,
                external: config.external,
                plugins,
            });
            const bundle = yield rollup_1.rollup(rollupOptions);
            const { code, umd } = yield handleBundle(bundle);
            if (config.esDest) {
                yield DevtoolHelper_1.DevtoolHelper.write(config.esDest, code);
            }
            if (config.umdDest) {
                yield DevtoolHelper_1.DevtoolHelper.write(config.umdDest, umd);
            }
            return code;
        }
        catch (e) {
            console.log(e);
        }
    });
}
exports.buildEnvoyPackage = buildEnvoyPackage;
