#!/usr/bin/env node
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const DevtoolHelper_1 = require("./util/DevtoolHelper");
const checkVersion_1 = require("./util/checkVersion");
const fs = __importStar(require("fs"));
const StyledText_1 = require("./util/StyledText");
const rollup_1 = require("rollup");
const qiniu_1 = require("./util/qiniu");
const path_1 = require("./config/path");
require('isomorphic-fetch');
const uglify = require('uglify-js');
const exec = require('child_process').execSync;
const path = require('path');
const md5File = require('md5-file');
const NODE_ENV = process.env.NODE_ENV;
const archiver = require('archiver');
let cache;
const date = new Date();
/**
 * Main function to build mesh package
 */
function buildMeshPackage() {
    const dateString = `${date.getFullYear()}${date.getMonth() + 1}${date.getDate()}${date.getHours()}${date.getMinutes()}${date.getSeconds()}`;
    process.env.BUILD_VERSION = dateString;
    // clean output files/dirs
    clean();
    // build the package
    build();
}
exports.buildMeshPackage = buildMeshPackage;
/**
 * Clean and recreate dist and zip directory
 */
function clean() {
    exec(`rm -rf ${path_1.paths.appDist} && mkdir ${path_1.paths.appDist}`);
    exec(`rm -rf ${path_1.paths.appZip} && mkdir ${path_1.paths.appZip}`);
    exec(`cp -r ${path_1.paths.appResource} ${path_1.paths.appDistResource}`);
}
/**
 * Bundle up scripts, archive and deploy it
 */
function build() {
    return __awaiter(this, void 0, void 0, function* () {
        const config = DevtoolHelper_1.DevtoolHelper.getConfig(cache);
        const packageBundleName = DevtoolHelper_1.DevtoolHelper.getPackageBundleName();
        if (!packageBundleName)
            return;
        try {
            const bundle = yield rollup_1.rollup(config);
            yield handleBundle(bundle);
            yield archive(packageBundleName);
            const isOutOfDate = yield checkVersion_1.checkVersion();
            if (isOutOfDate) {
                const str = `You are using Envoy v${isOutOfDate.local}, v${isOutOfDate.remote} is available`;
                console.log(StyledText_1.StyledText.blue(str));
            }
            if (NODE_ENV === 'production') {
                const filePath = path.join(path_1.paths.appZip, `${packageBundleName}.zip`);
                const bucket = 'mesh-package';
                const key = `${packageBundleName}.zip`;
                const fingerprint = md5File.sync(filePath);
                const message = yield qiniu_1.deploy(filePath, bucket, key, DevtoolHelper_1.appConfig, fingerprint);
                if (message) {
                    console.log(StyledText_1.StyledText.blue(message));
                }
            }
        }
        catch (e) {
            console.log(StyledText_1.StyledText.red(e));
        }
    });
}
/**
 * Handle the rollup bundle
 *
 * @param bundle - the bundle to be handled
 */
function handleBundle(bundle) {
    return __awaiter(this, void 0, void 0, function* () {
        // cache the bundle
        cache = bundle;
        const outputOptions = {
            format: 'iife',
        };
        const outputChunck = yield bundle.generate(outputOptions);
        const code = outputChunck.code;
        if (NODE_ENV === 'production') {
            const minified = uglify.minify(code, {
                fromString: true,
                output: {
                    ascii_only: true
                }
            }).code;
            return DevtoolHelper_1.DevtoolHelper.write(path_1.paths.appOutput, minified);
        }
        else {
            return DevtoolHelper_1.DevtoolHelper.write(path_1.paths.appOutput, code);
        }
    });
}
/**
 * Archive package bundle
 *
 * @param packageBundleName - name of package bundle to be archived
 */
function archive(packageBundleName) {
    return __awaiter(this, void 0, void 0, function* () {
        const output = fs.createWriteStream(path.join(path_1.paths.appZip, `${packageBundleName}.zip`));
        const archive = archiver('zip');
        archive.on('error', (err) => {
            throw err;
        });
        archive.pipe(output);
        archive.directory(path_1.paths.appDist, packageBundleName);
        return archive.finalize();
    });
}
