var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const StyledText_1 = require("./StyledText");
const fs = __importStar(require("fs"));
const path_1 = require("../config/path");
let appConfig;
exports.appConfig = appConfig;
try {
    exports.appConfig = appConfig = require(path_1.paths.appEnvoyConfig);
}
catch (e) {
    console.log(`File ${StyledText_1.StyledText.blue(path_1.paths.appEnvoyConfig)} doesnt exist`);
}
class DevtoolHelper {
    /**
     * Get configuration for building the Devtool
     *
     * @param cache - cached bundle
     */
    static getConfig(cache) {
        const NODE_ENV = process.env.NODE_ENV;
        let config;
        if (NODE_ENV === 'production') {
            config = require('../config/rollup.pro').default;
        }
        else if (NODE_ENV === 'test') {
            config = Object.assign({}, require('../config/rollup.test').default, {
                cache: cache
            });
        }
        else {
            config = Object.assign({}, require('../config/rollup.dev').default, {
                cache: cache
            });
        }
        return config;
    }
    /**
     * Gets package's bundle name
     */
    static getPackageBundleName() {
        const NODE_ENV = process.env.NODE_ENV;
        let packageBundleName;
        try {
            if (NODE_ENV === 'production') {
                if (!appConfig || !appConfig.name) {
                    console.log(StyledText_1.StyledText.red('No appConfig name provided in envoy.config.js'));
                    return "";
                }
                if (!appConfig || !appConfig.engineVersion) {
                    console.log(StyledText_1.StyledText.red('No engineVersion provided in envoy.config.js'));
                    return "";
                }
                if (!appConfig || !appConfig.accessKey) {
                    console.log(StyledText_1.StyledText.red('No accessKey provided in envoy.config.js'));
                    return "";
                }
                if (!appConfig || !appConfig.privateKey) {
                    console.log(StyledText_1.StyledText.red('No privateKey provided in envoy.config.js'));
                    return "";
                }
                packageBundleName = `${appConfig.name.replace(' ', '_')}_${appConfig.engineVersion.replace('.', '_')}_${process.env.BUILD_VERSION}`;
            }
            else {
                packageBundleName = appConfig.name || 'test';
            }
        }
        catch (error) {
            packageBundleName = 'test';
        }
        return packageBundleName;
    }
    /**
     * Write code to destination file
     *
     * @param dest - destination file as a string
     * @param code - code to be written into a file
     */
    static write(dest, code) {
        return __awaiter(this, void 0, void 0, function* () {
            return new Promise(function (resolve, reject) {
                fs.writeFile(dest, code, function (err) {
                    if (err)
                        return reject(err);
                    console.log(StyledText_1.StyledText.blue(dest) + ' ' + DevtoolHelper.getSize(code));
                    resolve();
                });
            });
        });
    }
    /**
     * Get size of a code in `kb`
     *
     * @param code - code to get size of
     */
    static getSize(code) {
        return (code.length / 1024).toFixed(2) + 'kb';
    }
}
exports.DevtoolHelper = DevtoolHelper;
